/* prefs.c
 * part of TeleGNOME, a GNOME app to view Teletext (Dutch)
 * This file constructs the preferences dialog.
 */

/*
** Copyright (C) 2000 Arjan Scherpenisse <acscherp@wins.uva.nl>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#include "prefs.h"

GtkWidget *construct_url_page();
/* not a good idea to have a 'misc' page, but i cant come up with a better name */
GtkWidget *construct_misc_page();

void prefs_cancel_cb(void);
void prefs_apply_cb(void);

void
set_tooltip(GtkWidget* w, const gchar* tip)
{
    GtkTooltips* t = gtk_tooltips_new();
     gtk_tooltips_set_tip (t, w, tip, NULL);
}


typedef struct _PrefsWindow {
    GnomePropertyBox *box;

    GtkWidget *page_entry;
    GtkWidget *sub_page_entry;

    GtkWidget *interval_entry;
    GtkWidget *proxy_entry;
} PrefsWindow;

static PrefsWindow *prefs_window;

void
prefs_show(void)
{
    GtkWidget *page;

    if (prefs_window != NULL) {
	gdk_window_show(GTK_WIDGET(prefs_window->box)->window);
	gdk_window_raise(GTK_WIDGET(prefs_window->box)->window);
    } else {
	prefs_window = g_malloc(sizeof(PrefsWindow));

	prefs_window->box = GNOME_PROPERTY_BOX (gnome_property_box_new());
	gtk_window_set_title (GTK_WINDOW(prefs_window->box), _("TeleGNOME: Preferences"));

	/* voeg een tab 'URLs' toe aan de preferences */
	page = construct_url_page();
	gtk_notebook_append_page(GTK_NOTEBOOK(prefs_window->box->notebook),
				 page,
				 gtk_label_new(_("URL's")));

	page = construct_misc_page();
	gtk_notebook_append_page(GTK_NOTEBOOK(prefs_window->box->notebook),
				 page,
				 gtk_label_new(_("Misc")));

	gtk_notebook_set_show_tabs (GTK_NOTEBOOK(prefs_window->box->notebook),
				    TRUE);
	gtk_notebook_set_show_border (GTK_NOTEBOOK(prefs_window->box->notebook),
				      TRUE);

	gtk_signal_connect (GTK_OBJECT (prefs_window->box), "apply",
			    GTK_SIGNAL_FUNC (prefs_apply_cb), NULL);
	gtk_signal_connect (GTK_OBJECT (prefs_window->box), "destroy",
			    GTK_SIGNAL_FUNC (prefs_cancel_cb), NULL);

	/* connect 'changed' signals */
	gtk_signal_connect_object(GTK_OBJECT(prefs_window->sub_page_entry), "changed",
				  GTK_SIGNAL_FUNC(gnome_property_box_changed),
				  GTK_OBJECT(prefs_window->box));
	
	gtk_signal_connect_object(GTK_OBJECT(prefs_window->page_entry), "changed",
				  GTK_SIGNAL_FUNC(gnome_property_box_changed),
				  GTK_OBJECT(prefs_window->box));

	gtk_signal_connect_object(GTK_OBJECT(prefs_window->interval_entry), "changed",
				  GTK_SIGNAL_FUNC(gnome_property_box_changed),
				  GTK_OBJECT(prefs_window->box));
	
	gtk_signal_connect_object(GTK_OBJECT(prefs_window->proxy_entry), "changed",
				  GTK_SIGNAL_FUNC(gnome_property_box_changed),
				  GTK_OBJECT(prefs_window->box));
	
	/* and, show them all */
	gtk_widget_show_all(GTK_WIDGET(prefs_window->box));
    }
}


GtkWidget *
construct_url_page() 
{
    GtkWidget *label;
    GtkWidget *vbox;

    vbox = gtk_vbox_new(FALSE, 0);

    label = gtk_label_new(_("Page URL"));
    gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 5);
    set_tooltip(label, _("URL to look up the teletext pages"));
    
    prefs_window->page_entry = gtk_entry_new_with_max_length(100);
    gtk_box_pack_start(GTK_BOX(vbox), prefs_window->page_entry, TRUE, TRUE, 5);

    label = gtk_label_new(_("Sub-page URL"));
    set_tooltip(label, _("URL to look up the teletext sub-pages"));
    gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 5);
    
    prefs_window->sub_page_entry = gtk_entry_new_with_max_length(100);
    gtk_box_pack_start(GTK_BOX(vbox), prefs_window->sub_page_entry, TRUE, TRUE, 5);


    gtk_entry_set_text(GTK_ENTRY(prefs_window->page_entry), 
		       gnome_config_get_string_with_default("/telegnome/URL/page_url=" DEFAULT_PAGE_URL,NULL));
    gtk_entry_set_text(GTK_ENTRY(prefs_window->sub_page_entry), 
		       gnome_config_get_string_with_default("/telegnome/URL/sub_page_url=" DEFAULT_SUB_PAGE_URL,NULL));
    
    gtk_widget_show_all(vbox);

    return vbox;
}

GtkWidget *
construct_misc_page()
{
    GtkWidget *vbox, *label, *entry, *proxy_label, *proxy_entry;
    GtkAdjustment *adj;

    g_assert(prefs_window != NULL);

    vbox = gtk_vbox_new(FALSE, 0);
    label = gtk_label_new(_("Paging interval"));
    set_tooltip(label, _("Specifies the interval for the auto-pager, in milliseconds."));
    gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 5);

    adj = GTK_ADJUSTMENT(gtk_adjustment_new(8000.0, 1000.0, 60000.0, 1000.0, 10.0, 0.0));
    entry = gtk_spin_button_new(adj, 0.5, 0);
    gtk_box_pack_start(GTK_BOX(vbox), entry, TRUE, TRUE, 5);

    gtk_spin_button_set_value(GTK_SPIN_BUTTON(entry), 
			      gnome_config_get_int_with_default("/telegnome/Paging/interval=" DEFAULT_INTERVAL,NULL));
    proxy_label = gtk_label_new(_("Proxy server"));
    gtk_box_pack_start(GTK_BOX(vbox), proxy_label, TRUE, TRUE, 5);
    proxy_entry = gtk_entry_new_with_max_length(100);
    gtk_box_pack_start(GTK_BOX(vbox), proxy_entry, TRUE, TRUE, 5);
    gtk_entry_set_text(GTK_ENTRY(proxy_entry), 
		       gnome_config_get_string_with_default("/telegnome/Proxy/http_proxy=" "",NULL));
    
    prefs_window->interval_entry = entry;
    prefs_window->proxy_entry = proxy_entry;
    return vbox;
}




/* pref window callbacks */
void
prefs_cancel_cb(void)
{
    gtk_widget_destroy(GTK_WIDGET(prefs_window->box));
    g_free(prefs_window);
    prefs_window = NULL;
}

void 
prefs_apply_cb(void)
{
   gnome_config_set_string("/telegnome/URL/page_url", gtk_entry_get_text(GTK_ENTRY(prefs_window->page_entry)));
   gnome_config_set_string("/telegnome/URL/sub_page_url", gtk_entry_get_text(GTK_ENTRY(prefs_window->sub_page_entry)));
   gnome_config_set_int("/telegnome/Paging/interval", 
			   gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(prefs_window->interval_entry)));
   gnome_config_set_string("/telegnome/Proxy/http_proxy", gtk_entry_get_text(GTK_ENTRY(prefs_window->proxy_entry)));
   gnome_config_sync();
}

